package no.ntnu.idatt1002.demo.data.recipes;

import java.io.*;
import java.util.List;
import java.util.Scanner;

//TODO: Refactor to make use of generics, which requires
// refactoring of more classes and write mor eunit tests and JavaDoc.
public class FileHandler {

    private static final String fileType = ".register";
    private static final String registerName = "RecipeRegister";
    private static final String filePath = "src/main/resources/recipes/";


    public static void writeRegister(RecipeRegister recipeRegister, String title) {
        if (recipeRegister == null) {
            throw new IllegalArgumentException("Only a valid register object can be written to file.");
        }

        try (FileWriter fileWriter = new FileWriter(filePath + title + fileType);) {
            recipeRegister.getRecipes().forEach((recipe) ->
                    {
                        try {
                            fileWriter.write(formatRecipe(recipe).toString());
                        } catch (IOException e) {
                            throw new RuntimeException(e);
                        }
                    }
            );

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static StringBuilder formatRecipe(Recipe recipe) {
        StringBuilder sb = new StringBuilder();
        sb.append("# ")
                .append(recipe.getName())
                .append("\n")
                .append(formatIngredientList(recipe.getIngredientList()))
                .append("\n")
                .append(recipe.getInstructions())
                .append("\n\n");
        return sb;
    }

    public static StringBuilder formatIngredientList(List<RecipeIngredient> ingredientList) {
        StringBuilder sb = new StringBuilder();

        ingredientList.forEach((ingredient) -> {
            sb.append("- ")
                    .append(ingredient.getFoodType())
                    .append(" | ")
                    .append(ingredient.getAmount())
                    .append(" | ")
                    .append(ingredient.getUnit())
                    .append("\n");
        });

        return sb;
    }


    //TODO: Midlertidig for testing.
    public static void readToTerminal(String title) throws FileNotFoundException {
        File file = new File(filePath + title + fileType);
        Scanner sc = new Scanner(file);
        while (sc.hasNext()) {
            System.out.println(sc.nextLine());
        }
    }


    public static RecipeRegister readRecipeRegister(String title) {
        File file = new File(filePath + title + fileType);

        RecipeRegister register = new RecipeRegister();

        try (Scanner sc = new Scanner(file)) {
            sc.useDelimiter("#");
            String line;

            while (sc.hasNext()) {
                line = sc.next();
                if (!line.isBlank()) {
                    register.addRecipe(readRecipe(line));
                }
            }
        } catch (FileNotFoundException e) {
            System.out.println("The file was not found.");
            return null;
        }

        return register;
    }

    public static Recipe readRecipe(String readRecipe) {

        Scanner sc = new Scanner(readRecipe);

        Recipe recipe;
        String instructions = "None";
        String recipeName = sc.nextLine().strip();
        StringBuilder sb = new StringBuilder();


        String line;
        recipe = new Recipe(recipeName, instructions);

        while (sc.hasNextLine()) {
            line = sc.nextLine();

            if (line.startsWith("-")) {
                String[] ingredientParts = line.split("\\|");

                FoodItem ingredientType = FoodItem.valueOf(ingredientParts[0].replaceFirst("-", "").strip());
                double ingredientAmount = Double.parseDouble(ingredientParts[1].strip());
                MeasuringUnit ingredientUnit = MeasuringUnit.valueOf(ingredientParts[2].strip());

                recipe.addIngredient(ingredientType, ingredientAmount, ingredientUnit);

            } else {
                sb.append(line).append("\n");
                if (!line.isBlank()) {
                   // sb.append(line).append("\n");
                }
            }
        }
        recipe.setInstructions(String.valueOf(sb).strip());

        return recipe;
    }


    // ================  Ingredients at hand ===============================

    public static void writeIngredientsAtHand(IngredientsAtHand ingredientsAtHand, String title) throws IOException {
        if (ingredientsAtHand == null) {
            throw new IllegalArgumentException("Only a valid register object can be written to file.");
        }

        StringBuilder sb = new StringBuilder();

        try (FileWriter fileWriter = new FileWriter(filePath + title + fileType);) {
            ingredientsAtHand.getIngredientsAtHand().forEach((ingredient) -> {
                sb.append(ingredient).append("\n");

            });
            try {
                fileWriter.write(String.valueOf(sb));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }


    public static IngredientsAtHand readIngredientsAtHand(String title) {
        File file = new File(filePath + title + fileType);

        IngredientsAtHand ingredientsAtHand = new IngredientsAtHand();

        try (Scanner sc = new Scanner(file)) {
            String line;

            while (sc.hasNext()) {
                line = sc.next();
                if (!line.isBlank()) {
                   ingredientsAtHand.addIngredient(FoodItem.valueOf(line));
                }
            }
        } catch (FileNotFoundException e) {
            System.out.println("The file was not found.");
            return null;
        }

        return ingredientsAtHand;
    }
}